#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <ArduinoJson.h>
#include <WebSocketsServer.h>

#define LED           D5
#define S0            D4
#define S1            D3
#define S3            D2
#define S2            D1
#define OUT           D0

#define SSID          "tvluke"
#define PASSWD        "tvluke"

WebSocketsServer webSocket(81);

boolean conectionEstablished = false;
int counter = 0;

void senseColor();
void startWebSocket();

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t lenght) { // When a WebSocket message is received
  switch (type) {
    case WStype_DISCONNECTED:             // if the websocket is disconnected
      Serial.printf("[%u] Disconnected!\n", num);
      conectionEstablished = false;
      break;
    case WStype_CONNECTED: {              // if a new websocket connection is established
        IPAddress ip = webSocket.remoteIP(num);
        conectionEstablished = true;
        Serial.printf("[%u] Connected from %d.%d.%d.%d url: %s\n", num, ip[0], ip[1], ip[2], ip[3], payload);
      }
      break;
    case WStype_TEXT:                     // if new text data is received
      Serial.printf("[%u] get Text: %s\n", num, payload);
      break;
  }
}

void setup() {
  pinMode(LED, OUTPUT);

  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(OUT, INPUT);

  // Setting frequency-scaling to 20%
  digitalWrite(S0,HIGH);
  digitalWrite(S1,LOW);

  Serial.begin(9600);

  startWebSocket();

  WiFi.mode(WIFI_AP);
  WiFi.disconnect();
  WiFi.softAP(SSID);
  Serial.println();
  Serial.print("Server IP address: ");
  Serial.println(WiFi.softAPIP());
  Serial.print("Server MAC address: ");
  Serial.println(WiFi.softAPmacAddress());
}

void startWebSocket() { // Start a WebSocket server
  webSocket.begin();                          // start the websocket server
  webSocket.onEvent(webSocketEvent);          // if there's an incomming websocket message, go to function 'webSocketEvent'
  Serial.println("WebSocket server started.");
}

void loop() {
  webSocket.loop();
  //server.handleClient();
  counter = counter + 1;
  if(conectionEstablished && counter > 50000) {
    counter = 0;
    senseColor();
  }
}

void senseColor() {
  digitalWrite(LED,HIGH);
  Serial.println("SENSE");

  // Setting red filtered photodiodes to be read
  digitalWrite(S2,LOW);
  digitalWrite(S3,LOW);
  // Reading the output frequency
  unsigned long frequency = pulseIn(OUT, LOW);
  unsigned long red = frequency;
  delay(100);

  // Setting Green filtered photodiodes to be read
  digitalWrite(S2,HIGH);
  digitalWrite(S3,HIGH);
  frequency = pulseIn(OUT, LOW);
  unsigned long green = frequency;
  delay(100);

  // Setting Blue filtered photodiodes to be read
  digitalWrite(S2,LOW);
  digitalWrite(S3,HIGH);
  frequency = pulseIn(OUT, LOW);
  unsigned long blue = frequency;

  if(red > 255) {
     red = 255;
   }
   if(green > 255) {
     green = 255;
   }
   if(blue > 255) {
     blue = 255;
   }

  int redX = (int) abs(red-255);
  int greenX = (int) abs(green-255);
  int blueX = (int) abs(blue-255);

  String bla = "";
  bla.concat(redX);
  bla.concat(" ");
  bla.concat(greenX);
  bla.concat(" ");
  bla.concat(blueX);
  webSocket.sendTXT(0, bla);
  digitalWrite(LED,LOW);
}
